@@ -96,9 +96,8 @@ typedef enum
 static PyObject *
 compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
 {
-  const char *source;
   const char *mode = "default";
-  int source_size, dest_size;
+  int dest_size, total_size;
   int acceleration = 1, compression = 0;
   int store_size = 1;
   PyObject *py_dest;
@@ -114,12 +113,46 @@ compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
     NULL
   };
 
+#if IS_PY3
+  PyObject * py_source;
+  Py_ssize_t source_size;
+  char * source;
+  if (!PyArg_ParseTupleAndKeywords (args, kwargs, "O|siii", argnames,
+                                    &py_source,
+                                    &mode, &store_size, &acceleration, &compression))
+    {
+      return NULL;
+    }
+  if (PyByteArray_Check(py_source))
+    {
+      source = PyByteArray_AsString(py_source);
+      if (source == NULL)
+        {
+          PyErr_SetString (PyExc_ValueError, "Failed to access source bytearray object");
+          return NULL;
+        }
+      source_size = PyByteArray_GET_SIZE(py_source);
+    }
+  else
+    {
+      source = PyBytes_AsString(py_source);
+      if (source == NULL)
+        {
+          PyErr_SetString (PyExc_ValueError, "Failed to access source object");
+          return NULL;
+        }
+      source_size = PyBytes_GET_SIZE(py_source);
+    }
+#else
+  const char *source;
+  int source_size;
   if (!PyArg_ParseTupleAndKeywords (args, kwargs, "s#|siii", argnames,
                                     &source, &source_size,
                                     &mode, &store_size, &acceleration, &compression))
     {
       return NULL;
     }
+#endif
 
   if (!strncmp (mode, "default", sizeof ("default")))
     {
@@ -145,19 +178,29 @@ compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
 
   if (store_size)
     {
-      py_dest = PyBytes_FromStringAndSize (NULL, dest_size + hdr_size);
+      total_size = dest_size + hdr_size;
     }
   else
     {
-      py_dest = PyBytes_FromStringAndSize (NULL, dest_size);
+      total_size = dest_size + hdr_size;
     }
 
+#if IS_PY3
+  py_dest = PyByteArray_FromStringAndSize (NULL, total_size);
+#else
+  py_dest = PyBytes_FromStringAndSize (NULL, total_size);
+#endif
+
   if (py_dest == NULL)
     {
       return PyErr_NoMemory();
     }
 
+#if IS_PY3
+  dest = PyByteArray_AS_STRING (py_dest);
+#else
   dest = PyBytes_AS_STRING (py_dest);
+#endif
 
   Py_BEGIN_ALLOW_THREADS
 
@@ -190,7 +233,7 @@ compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
   if (output_size <= 0)
     {
       Py_BLOCK_THREADS
-      PyErr_SetString (PyExc_ValueError, "Compression failed");
+        PyErr_SetString (PyExc_ValueError, "Compression failed");
       Py_CLEAR (py_dest);
       return NULL;
     }
@@ -205,7 +248,11 @@ compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
   /* Resizes are expensive; tolerate some slop to avoid. */
   if (output_size < (dest_size / 4) * 3)
     {
+#if IS_PY3
+      PyByteArray_Resize (py_dest, output_size);
+#else
       _PyBytes_Resize (&py_dest, output_size);
+#endif
     }
   else
     {
@@ -218,10 +265,10 @@ compress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
 static PyObject *
 decompress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
 {
-  const char *source, *source_start;
+  const char * source_start;
   PyObject *py_dest;
   char *dest;
-  int source_size, output_size;
+  int output_size;
   size_t dest_size;
   int uncompressed_size = -1;
   static char *argnames[] = {
@@ -230,12 +277,44 @@ decompress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
     NULL
   };
 
+#if IS_PY3
+  PyObject * py_source;
+  Py_ssize_t source_size;
+  char * source;
+  if (!PyArg_ParseTupleAndKeywords (args, kwargs, "O|i", argnames,
+                                    &py_source, &uncompressed_size))
+    {
+      return NULL;
+    }
+  if (PyByteArray_Check(py_source))
+    {
+      source = PyByteArray_AsString(py_source);
+      if (source == NULL)
+        {
+          PyErr_SetString (PyExc_ValueError, "Failed to access source bytearray object");
+          return NULL;
+        }
+      source_size = PyByteArray_Size(py_source);
+    }
+  else
+    {
+      source = PyBytes_AsString(py_source);
+      if (source == NULL)
+        {
+          PyErr_SetString (PyExc_ValueError, "Failed to access source object");
+          return NULL;
+        }
+      source_size = PyBytes_Size(py_source);
+    }
+#else
+  const char *source;
+  int source_size = 0;
   if (!PyArg_ParseTupleAndKeywords (args, kwargs, "s#|i", argnames,
                                     &source, &source_size, &uncompressed_size))
     {
       return NULL;
     }
-
+#endif
   if (uncompressed_size > 0)
     {
       dest_size = uncompressed_size;
@@ -260,13 +339,21 @@ decompress (PyObject * Py_UNUSED (self), PyObject * args, PyObject * kwargs)
       return NULL;
     }
 
+#if IS_PY3
+  py_dest = PyByteArray_FromStringAndSize (NULL, dest_size);
+#else
   py_dest = PyBytes_FromStringAndSize (NULL, dest_size);
+#endif
   if (py_dest == NULL)
     {
       return PyErr_NoMemory();
     }
 
+#if IS_PY3
+  dest = PyByteArray_AS_STRING (py_dest);
+#else
   dest = PyBytes_AS_STRING (py_dest);
+#endif
 
   Py_BEGIN_ALLOW_THREADS
 
@@ -297,7 +384,7 @@ PyDoc_STRVAR(compress__doc,
              "Compress source, returning the compressed data as a string.\n" \
              "Raises an exception if any error occurs.\n\n"             \
              "Args:\n"                                                  \
-             "    source (str): Data to compress\n"                     \
+             "    source (str, bytes or bytearray): Data to compress\n"                     \
              "    mode (str): If 'default' or unspecified use the default LZ4\n" \
              "        compression mode. Set to 'fast' to use the fast compression\n" \
              "        LZ4 mode at the expense of compression. Set to\n" \
@@ -311,23 +398,25 @@ PyDoc_STRVAR(compress__doc,
              "        argument specifies the compression. Valid values are between\n" \
              "        0 and 16. Values between 4-9 are recommended, and 0 is the\n" \
              "        default.\n\n"                                     \
-             "    store_size (bool): If True (the default) then the size of the" \
-             "        uncompressed data is stored at the start of the compressed" \
-             "        block."                                            \
+             "    store_size (bool): If True (the default) then the size of the\n" \
+             "        uncompressed data is stored at the start of the compressed\n" \
+             "        block.\n\n"                                       \
              "Returns:\n"                                               \
-             "    str: Compressed data\n");
+             "    str or bytearray: Compressed data. For Python 2 a str is returned\n" \
+             "        and for Python 3 a bytearray is returned\n");
 
 PyDoc_STRVAR(decompress__doc,
-             "decompress(source)\n\n"                                   \
+             "decompress(source, uncompressed_size=-1)\n\n"                                 \
              "Decompress source, returning the uncompressed data as a string.\n" \
              "Raises an exception if any error occurs.\n\n"             \
              "Args:\n"                                                  \
-             "    source (str): Data to decompress\n\n"                 \
-             "    uncompressed_size (int): If not specified, the uncompressed data" \
-             "        size is read from the start of the source block. If specified," \
-             "        it is assumed that the full source data is compressed data."
-             "Returns:\n"                                               \
-             "    str: decompressed data\n");
+             "    source (str, bytes or bytearray): Data to decompress\n\n"                 \
+             "    uncompressed_size (int): If not specified or < 0, the uncompressed data\n" \
+             "        size is read from the start of the source block. If specified,\n" \
+             "        it is assumed that the full source data is compressed data.\n\n" \
+             "Returns:\n"                                             \
+             "    str or bytearray: Decompressed data. For Python 2 a str is returned\n" \
+             "        and for Python 3 a bytearray is returned\n");
 
 PyDoc_STRVAR(lz4block__doc,
              "A Python wrapper for the LZ4 block protocol"
